#include <windows.h>
#include <tchar.h>
#include <memory.h>
#include <commctrl.h>

#include "resource.h"
#include "ppongo.h"

DWORD	v_BaudRates[] = 
{
	CBR_110, CBR_300, CBR_600, CBR_1200, CBR_2400, CBR_4800, CBR_9600, CBR_14400, CBR_19200, CBR_38400, CBR_56000, CBR_57600, CBR_115200, CBR_128000, CBR_256000 	
};

#define NUM_BAUD_RATES	(sizeof(v_BaudRates)/sizeof(DWORD))


LRESULT CALLBACK CommDlgProc (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	DCB		PortDCB;
	DWORD	i;
	TCHAR	szTempStr[80];
	HWND	hCtrlWnd;
	BOOL	bWasClose=FALSE;

    switch (message) 
	{
		case WM_INITDIALOG :

			wsprintf (szTempStr, TEXT("Communications settings for '%s'"), v_szDevName);
			SetWindowText (GetDlgItem (hDlg, IDC_PORTLABEL), szTempStr);

			if(hPort == INVALID_HANDLE_VALUE)
			{
				bWasClose=TRUE;
				if(!OpenPort(v_szDevName))
					return FALSE;
			}
			else
				bWasClose=FALSE;
			
			PortDCB.DCBlength = sizeof(DCB);
			GetCommState(hPort, &PortDCB);

			if(bWasClose==TRUE)
				ClosePort();

			hCtrlWnd = GetDlgItem (hDlg, IDC_BAUDRATE);
			
			for (i=0; i < NUM_BAUD_RATES; i++) 
			{
				wsprintf (szTempStr, TEXT("%d"), v_BaudRates[i]);
				SendMessage (hCtrlWnd, CB_ADDSTRING,0, (LPARAM)szTempStr);
			}
			
			wsprintf (szTempStr, TEXT("%d"), PortDCB.BaudRate);
			SendMessage (hCtrlWnd, CB_SETCURSEL, SendMessage (hCtrlWnd, CB_FINDSTRINGEXACT, 0, (LPARAM)szTempStr), 0);

			hCtrlWnd = GetDlgItem (hDlg, IDC_DATABITS);
			
			for (i=4; i <= 8; i++) 
			{
				wsprintf (szTempStr, TEXT("%d"), i);
				SendMessage (hCtrlWnd, CB_ADDSTRING, 0, (LPARAM)szTempStr);
			}
			
			wsprintf (szTempStr, TEXT("%d"), PortDCB.ByteSize);
			SendMessage (hCtrlWnd, CB_SETCURSEL, SendMessage (hCtrlWnd, CB_FINDSTRINGEXACT, 0, (LPARAM)szTempStr),0);
		
			// Stop Bits.
			hCtrlWnd = GetDlgItem (hDlg, IDC_STOPBITS);
			SendMessage (hCtrlWnd, CB_ADDSTRING, 0, (LPARAM)TEXT("1"));
			SendMessage (hCtrlWnd, CB_ADDSTRING, 0, (LPARAM)TEXT("1.5"));
			SendMessage (hCtrlWnd, CB_ADDSTRING, 0, (LPARAM)TEXT("2"));
			SendMessage (hCtrlWnd, CB_SETCURSEL, (WPARAM)PortDCB.StopBits, 0);
			
			// Parity
			hCtrlWnd = GetDlgItem (hDlg, IDC_PARITY);
			SendMessage (hCtrlWnd, CB_ADDSTRING, 0, (LPARAM)TEXT("None"));
			SendMessage (hCtrlWnd, CB_ADDSTRING, 0, (LPARAM)TEXT("Odd"));
			SendMessage (hCtrlWnd, CB_ADDSTRING, 0, (LPARAM)TEXT("Even"));
			SendMessage (hCtrlWnd, CB_ADDSTRING, 0, (LPARAM)TEXT("Mark"));
			SendMessage (hCtrlWnd, CB_ADDSTRING, 0, (LPARAM)TEXT("Space"));
			SendMessage (hCtrlWnd, CB_SETCURSEL, (WPARAM)PortDCB.Parity, 0);
			
			// Flow Control
			hCtrlWnd = GetDlgItem (hDlg, IDC_FLOWCTRL);
			SendMessage (hCtrlWnd, CB_ADDSTRING, 0, (LPARAM)TEXT("None"));
			SendMessage (hCtrlWnd, CB_ADDSTRING, 0, (LPARAM)TEXT("RTS/CTS"));
			SendMessage (hCtrlWnd, CB_ADDSTRING, 0, (LPARAM)TEXT("XON/XOFF"));
			
			i = 0;
			
			if (PortDCB.fOutX) 
			{
				i = 2;
			} 
			else if (PortDCB.fOutxCtsFlow) 
			{
				i = 1;
			}

			SendMessage (hCtrlWnd, CB_SETCURSEL, (WPARAM)i, 0);

			// DSR Sensitivity
			SendMessage (GetDlgItem (hDlg, IDC_DSR_SENS), BM_SETCHECK,(WPARAM)(PortDCB.fDsrSensitivity ? 1 : 0), (LPARAM)0);
			return FALSE;

		case WM_COMMAND :
			
			switch (LOWORD(wParam)) 
			{
				case IDCANCEL :
					EndDialog (hDlg, FALSE);
					break;
				case IDOK :

					// Save the settings
					
					if(hPort == INVALID_HANDLE_VALUE)
					{
						bWasClose=TRUE;
						if(!OpenPort(v_szDevName))
							return FALSE;
					}
					else
						bWasClose=FALSE;

					PortDCB.DCBlength = sizeof(DCB);
					GetCommState(hPort, &PortDCB);

					GetWindowText (GetDlgItem (hDlg, IDC_BAUDRATE), szTempStr, sizeof (szTempStr)/sizeof(TCHAR));
					
					PortDCB.BaudRate = _ttol (szTempStr);

					GetWindowText (GetDlgItem (hDlg, IDC_DATABITS), szTempStr, sizeof(szTempStr)/sizeof(TCHAR));
					
					PortDCB.ByteSize = (BYTE)_ttol (szTempStr);
					
					PortDCB.StopBits = (BYTE)SendMessage (GetDlgItem (hDlg, IDC_STOPBITS), CB_GETCURSEL, (WPARAM)0, (LPARAM)0);
					
					PortDCB.Parity = (BYTE)SendMessage (GetDlgItem (hDlg, IDC_PARITY), CB_GETCURSEL, (WPARAM)0, (LPARAM)0);
					
					i = SendMessage (GetDlgItem (hDlg, IDC_FLOWCTRL), CB_GETCURSEL, (WPARAM)0, (LPARAM)0);

					// Set to no flow control
					PortDCB.fOutX = FALSE;
					PortDCB.fInX = FALSE;
					PortDCB.fOutxCtsFlow = FALSE;
					PortDCB.fRtsControl = RTS_CONTROL_ENABLE;

					if (i == 2) 
					{
						PortDCB.fOutX = TRUE;
						PortDCB.fInX = TRUE;
					} 
					else if (i == 1) 
					{
						PortDCB.fOutxCtsFlow = TRUE;
						PortDCB.fRtsControl = RTS_CONTROL_HANDSHAKE;
					}

					
					// DSR Sensitivity
					i = SendMessage (GetDlgItem (hDlg, IDC_DSR_SENS), BM_GETCHECK, (WPARAM)0, (LPARAM)0);

					PortDCB.fDsrSensitivity = (i) ? 1 : 0;

					SetCommState (hPort, &PortDCB);
					
					if(bWasClose==TRUE)
						ClosePort();

//					PrintDCB (hPort);
					EndDialog (hDlg, TRUE);
					break;
			}
			break;
		case WM_DESTROY :
			break;
		default :
			return (FALSE);
	}
	return TRUE;
}

